@@ -23,6 +23,7 @@ import bytecode as _bytecode
 from bytecode.concrete import ConcreteInstr
 from bytecode.flags import CompilerFlags
 from bytecode.instr import UNSET, Instr, Label, SetLineno, TryBegin, TryEnd
+from bytecode.utils import PY310, PY311, PY313
 
 T = TypeVar("T", bound="BasicBlock")
 U = TypeVar("U", bound="ControlFlowGraph")
@@ -57,7 +58,7 @@ class BasicBlock(_bytecode._InstrList[Union[Instr, SetLineno, TryBegin, TryEnd]]
                     isinstance(self[i], Instr) for i in range(index, len(self))
                 ):
                     raise ValueError(
-                        "Only the last instruction of a basic " "block can be a jump"
+                        "Only the last instruction of a basic block can be a jump"
                     )
 
                 if not isinstance(instr.arg, BasicBlock):
@@ -78,12 +79,10 @@ class BasicBlock(_bytecode._InstrList[Union[Instr, SetLineno, TryBegin, TryEnd]]
     @overload
     def __getitem__(
         self, index: SupportsIndex
-    ) -> Union[Instr, SetLineno, TryBegin, TryEnd]:
-        ...
+    ) -> Union[Instr, SetLineno, TryBegin, TryEnd]: ...
 
     @overload
-    def __getitem__(self: T, index: slice) -> T:
-        ...
+    def __getitem__(self: T, index: slice) -> T: ...
 
     def __getitem__(self, index):
         value = super().__getitem__(index)
@@ -359,10 +358,13 @@ class _StackSizeComputer:
                 if instr.is_uncond_jump():
                     # Check for TryEnd after the final instruction which is possible
                     # TryEnd being only pseudo instructions
-                    if te := self.block.get_trailing_try_end(i):
-                        # TryBegin cannot be nested
-                        assert te.entry is self._current_try_begin
-
+                    # TryBegin cannot be nested so a TryEnd should always match the
+                    # current try begin. However inside the CFG some blocks may
+                    # start with a TryEnd relevant only when reaching this block
+                    # through a particular jump. So we are lenient here.
+                    if (
+                        te := self.block.get_trailing_try_end(i)
+                    ) and te.entry is self._current_try_begin:
                         assert isinstance(te.entry.target, BasicBlock)
                         yield from self._compute_exception_handler_stack_usage(
                             te.entry.target,
@@ -445,7 +447,7 @@ class _StackSizeComputer:
     def _is_stacksize_computation_relevant(
         self, block_id: int, fingerprint: Tuple[int, Optional[bool]]
     ) -> bool:
-        if sys.version_info >= (3, 11):
+        if PY311:
             # The computation is relevant if the block was not visited previously
             # with the same starting size and exception handler status than the
             # one in use
@@ -478,7 +480,7 @@ class ControlFlowGraph(_bytecode.BaseBytecode):
         try:
             return self._block_index[id(block)]
         except KeyError:
-            raise ValueError("the block is not part of this bytecode")
+            raise ValueError(f"the block {block} is not part of this bytecode")  # noqa
 
     def _add_block(self, block: BasicBlock) -> None:
         block_index = len(self._blocks)
@@ -521,10 +523,15 @@ class ControlFlowGraph(_bytecode.BaseBytecode):
         # Starting with Python 3.10, generator and coroutines start with one object
         # on the stack (None, anything is an error).
         initial_stack_size = 0
-        if sys.version_info >= (3, 10) and self.flags & (
-            CompilerFlags.GENERATOR
-            | CompilerFlags.COROUTINE
-            | CompilerFlags.ASYNC_GENERATOR
+        if (
+            not PY313  # under 3.13+ RETURN_GENERATOR make this explicit
+            and PY310
+            and self.flags
+            & (
+                CompilerFlags.GENERATOR
+                | CompilerFlags.COROUTINE
+                | CompilerFlags.ASYNC_GENERATOR
+            )
         ):
             initial_stack_size = 1
 
@@ -639,7 +646,7 @@ class ControlFlowGraph(_bytecode.BaseBytecode):
         return instructions
 
     def __eq__(self, other: Any) -> bool:
-        if type(self) != type(other):
+        if type(self) is not type(other):
             return False
 
         if self.argnames != other.argnames:
@@ -660,12 +667,10 @@ class ControlFlowGraph(_bytecode.BaseBytecode):
         return iter(self._blocks)
 
     @overload
-    def __getitem__(self, index: Union[int, BasicBlock]) -> BasicBlock:
-        ...
+    def __getitem__(self, index: Union[int, BasicBlock]) -> BasicBlock: ...
 
     @overload
-    def __getitem__(self: U, index: slice) -> U:
-        ...
+    def __getitem__(self: U, index: slice) -> U: ...
 
     def __getitem__(self, index):
         if isinstance(index, BasicBlock):
@@ -678,8 +683,8 @@ class ControlFlowGraph(_bytecode.BaseBytecode):
         block = self._blocks[index]
         del self._blocks[index]
         del self._block_index[id(block)]
-        for index in range(index, len(self)):
-            block = self._blocks[index]
+        for i in range(index, len(self)):
+            block = self._blocks[i]
             self._block_index[id(block)] -= 1
 
     def split_block(self, block: BasicBlock, index: int) -> BasicBlock:
@@ -955,8 +960,8 @@ class ControlFlowGraph(_bytecode.BaseBytecode):
         # TryEnd/TryBegin pair which share the same target.
         # In each case, we store the value found in the CFG and the value
         # inserted in the bytecode.
-        last_try_begin: tuple[TryBegin, TryBegin] | None = None
-        last_try_end: tuple[TryEnd, TryEnd] | None = None
+        last_try_begin: Tuple[TryBegin, TryBegin] | None = None
+        last_try_end: Tuple[TryEnd, TryEnd] | None = None
 
         for block in self:
             if id(block) in used_blocks:
