@@ -3,6 +3,7 @@ import asyncio
 import contextlib
 import inspect
 import io
+import opcode
 import sys
 import textwrap
 import types
@@ -19,9 +20,9 @@ from bytecode import (
     dump_bytecode,
 )
 from bytecode.concrete import OFFSET_AS_INSTRUCTION
+from bytecode.utils import PY311, PY313
 
-from . import TestCase
-from . import disassemble as _disassemble
+from . import TestCase, disassemble as _disassemble
 
 
 def disassemble(
@@ -612,6 +613,7 @@ class BytecodeBlocksFunctionalTests(TestCase):
         block0.extend(
             [
                 Instr("LOAD_FAST", "x", lineno=4),
+                *([Instr("TO_BOOL", lineno=4)] if PY313 else []),
                 Instr(
                     "POP_JUMP_FORWARD_IF_FALSE"
                     if (3, 12) > sys.version_info >= (3, 11)
@@ -633,20 +635,45 @@ class BytecodeBlocksFunctionalTests(TestCase):
             ]
         )
 
-        if sys.version_info >= (3, 11):
-            # jump is relative not absolute
-            expected = (
-                b"|\x05" b"r\x02" b"|\x00" b"}\x05" b"d\x01" b"}\x05" b"|\x05" b"S\x00"
+        if PY313:
+            expected = bytes(
+                (
+                    opcode.opmap["LOAD_FAST"],
+                    5,
+                    opcode.opmap["TO_BOOL"],
+                    0,
+                    0,
+                    0,
+                    0,
+                    0,
+                    0,
+                    0,
+                    opcode.opmap["POP_JUMP_IF_FALSE"],
+                    2,
+                    0,
+                    0,
+                    opcode.opmap["LOAD_FAST"],
+                    0,
+                    opcode.opmap["STORE_FAST"],
+                    5,
+                    opcode.opmap["LOAD_CONST"],
+                    1,
+                    opcode.opmap["STORE_FAST"],
+                    5,
+                    opcode.opmap["LOAD_FAST"],
+                    5,
+                    opcode.opmap["RETURN_VALUE"],
+                    0,
+                )
             )
+        elif PY311:
+            # jump is relative not absolute
+            expected = b"|\x05r\x02|\x00}\x05d\x01}\x05|\x05S\x00"
         elif OFFSET_AS_INSTRUCTION:
             # The argument of the jump is divided by 2
-            expected = (
-                b"|\x05" b"r\x04" b"|\x00" b"}\x05" b"d\x01" b"}\x05" b"|\x05" b"S\x00"
-            )
+            expected = b"|\x05r\x04|\x00}\x05d\x01}\x05|\x05S\x00"
         else:
-            expected = (
-                b"|\x05" b"r\x08" b"|\x00" b"}\x05" b"d\x01" b"}\x05" b"|\x05" b"S\x00"
-            )
+            expected = b"|\x05r\x08|\x00}\x05d\x01}\x05|\x05S\x00"
 
         code = bytecode.to_code()
         self.assertEqual(code.co_consts, (None, 3))
@@ -785,7 +812,7 @@ class CFGStacksizeComputationTests(TestCase):
             try:
                 return args[0]
             finally:
-                return 2
+                return 2  # noqa
 
         self.check_stack_size(test)
 
@@ -822,7 +849,7 @@ class CFGStacksizeComputationTests(TestCase):
                 pass
             except Exception:
                 try:
-                    assert False
+                    raise RuntimeError
                 except Exception:
                     return 2
                 finally:
